@@ -137,8 +137,7 @@ int main(int argc, char **argv)
 
 	if (count > intel_get_total_ram_mb() * 9 / 10) {
 		count = intel_get_total_ram_mb() * 9 / 10;
-		fprintf(stderr, "not enough RAM to run test, reducing buffer count\n");
-		return 77;
+		printf("not enough RAM to run test, reducing buffer count\n");
 	}
 
 	printf("Using %d 1MiB buffers\n", count);
