@@ -312,7 +312,8 @@ render_copy(int fd,
 	exec.num_cliprects = 0;
 	exec.cliprects_ptr = 0;
 	exec.flags = 0;
-	exec.rsvd1 = exec.rsvd2 = 0;
+	i915_execbuffer2_set_context_id(exec, 0);
+	exec.rsvd2 = 0;
 
 	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, &exec);
 	while (ret && errno == EBUSY) {
@@ -389,7 +390,8 @@ static void blt_copy(int fd, uint32_t dst, uint32_t src)
 	exec.num_cliprects = 0;
 	exec.cliprects_ptr = 0;
 	exec.flags = 0;
-	exec.rsvd1 = exec.rsvd2 = 0;
+	i915_execbuffer2_set_context_id(exec, 0);
+	exec.rsvd2 = 0;
 
 	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_EXECBUFFER2, &exec);
 	while (ret && errno == EBUSY) {
