@@ -58,16 +58,12 @@
 #include <sys/mman.h>
 #include <sys/ioctl.h>
 
-#include "xf86drm.h"
-#include "xf86drmMode.h"
 #include "i915_drm.h"
 #include "drmtest.h"
 #include "testdisplay.h"
 
-#if defined(DRM_IOCTL_MODE_ADDFB2) && defined(DRM_I915_SET_SPRITE_COLORKEY)
-#define TEST_PLANES 1
-#include "drm_fourcc.h"
-#endif
+#include <stdlib.h>
+#include <signal.h>
 
 drmModeRes *resources;
 int drm_fd, modes;
@@ -75,6 +71,7 @@ int dump_info = 0, test_all_modes =0, test_preferred_mode = 0, force_mode = 0,
 	test_plane, enable_tiling;
 int sleep_between_modes = 5;
 uint32_t depth = 24, stride, bpp;
+int qr_code = 0;
 
 drmModeModeInfo force_timing;
 
@@ -158,25 +155,6 @@ struct connector {
 	int pipe;
 };
 
-static void dump_mode(drmModeModeInfo *mode)
-{
-	printf("  %s %d %d %d %d %d %d %d %d %d 0x%x 0x%x %d\n",
-	       mode->name,
-	       mode->vrefresh,
-	       mode->hdisplay,
-	       mode->hsync_start,
-	       mode->hsync_end,
-	       mode->htotal,
-	       mode->vdisplay,
-	       mode->vsync_start,
-	       mode->vsync_end,
-	       mode->vtotal,
-	       mode->flags,
-	       mode->type,
-	       mode->clock);
-}
-
-
 static void dump_connectors_fd(int drmfd)
 {
 	int i, j;
@@ -216,7 +194,7 @@ static void dump_connectors_fd(int drmfd)
 		printf("  name refresh (Hz) hdisp hss hse htot vdisp "
 		       "vss vse vtot flags type clock\n");
 		for (j = 0; j < connector->count_modes; j++)
-			dump_mode(&connector->modes[j]);
+			kmstest_dump_mode(&connector->modes[j]);
 
 		drmModeFreeConnector(connector);
 	}
@@ -246,7 +224,7 @@ static void dump_crtcs_fd(int drmfd)
 		       crtc->buffer_id,
 		       crtc->x, crtc->y,
 		       crtc->width, crtc->height);
-		dump_mode(&crtc->mode);
+		kmstest_dump_mode(&crtc->mode);
 
 		drmModeFreeCrtc(crtc);
 	}
@@ -255,46 +233,6 @@ static void dump_crtcs_fd(int drmfd)
 	drmModeFreeResources(mode_resources);
 }
 
-
-#ifdef TEST_PLANES
-static void dump_planes(void)
-{
-	drmModePlaneRes *plane_resources;
-	drmModePlane *ovr;
-	int i;
-
-	plane_resources = drmModeGetPlaneResources(drm_fd);
-	if (!plane_resources) {
-		fprintf(stderr, "drmModeGetPlaneResources dump failed: %s\n",
-			strerror(errno));
-		return;
-	}
-
-	printf("Planes:\n");
-	printf("id\tcrtc\tfb\tCRTC x,y\tx,y\tgamma size\n");
-	for (i = 0; i < plane_resources->count_planes; i++) {
-		ovr = drmModeGetPlane(drm_fd, plane_resources->planes[i]);
-		if (!ovr) {
-			fprintf(stderr, "drmModeGetPlane failed: %s\n",
-				strerror(errno));
-			continue;
-		}
-
-		printf("%d\t%d\t%d\t%d,%d\t\t%d,%d\t%d\n",
-		       ovr->plane_id, ovr->crtc_id, ovr->fb_id,
-		       ovr->crtc_x, ovr->crtc_y, ovr->x, ovr->y,
-		       ovr->gamma_size);
-
-		drmModeFreePlane(ovr);
-	}
-	printf("\n");
-
-	return;
-}
-#else
-static void dump_planes(void) { return; }
-#endif
-
 static void connector_find_preferred_mode(struct connector *c)
 {
 	drmModeConnector *connector;
@@ -386,101 +324,6 @@ static void connector_find_preferred_mode(struct connector *c)
 	c->connector = connector;
 }
 
-static cairo_surface_t *
-allocate_surface(int fd, int width, int height, uint32_t depth,
-		 uint32_t *handle, int tiled)
-{
-	cairo_format_t format;
-	struct drm_i915_gem_set_tiling set_tiling;
-	int size;
-
-	if (tiled) {
-		int v;
-
-		/* Round the tiling up to the next power-of-two and the
-		 * region up to the next pot fence size so that this works
-		 * on all generations.
-		 *
-		 * This can still fail if the framebuffer is too large to
-		 * be tiled. But then that failure is expected.
-		 */
-
-		v = width * bpp / 8;
-		for (stride = 512; stride < v; stride *= 2)
-			;
-
-		v = stride * height;
-		for (size = 1024*1024; size < v; size *= 2)
-			;
-	} else {
-		/* Scan-out has a 64 byte alignment restriction */
-		stride = (width * (bpp / 8) + 63) & ~63;
-		size = stride * height;
-	}
-
-	switch (depth) {
-	case 16:
-		format = CAIRO_FORMAT_RGB16_565;
-		break;
-	case 24:
-		format = CAIRO_FORMAT_RGB24;
-		break;
-#if 0
-	case 30:
-		format = CAIRO_FORMAT_RGB30;
-		break;
-#endif
-	case 32:
-		format = CAIRO_FORMAT_ARGB32;
-		break;
-	default:
-		fprintf(stderr, "bad depth %d\n", depth);
-		return NULL;
-	}
-
-	*handle = gem_create(fd, size);
-
-	if (tiled) {
-		set_tiling.handle = *handle;
-		set_tiling.tiling_mode = I915_TILING_X;
-		set_tiling.stride = stride;
-		if (ioctl(fd, DRM_IOCTL_I915_GEM_SET_TILING, &set_tiling)) {
-			fprintf(stderr, "set tiling failed: %s (stride=%d, size=%d)\n",
-				strerror(errno), stride, size);
-			return NULL;
-		}
-	}
-
-	fb_ptr = gem_mmap(fd, *handle, size, PROT_READ | PROT_WRITE);
-
-	return cairo_image_surface_create_for_data((unsigned char *)fb_ptr,
-						   format, width, height,
-						   stride);
-}
-
-enum corner {
-	topleft,
-	topright,
-	bottomleft,
-	bottomright,
-};
-
-static void
-paint_color_gradient(cairo_t *cr, int x, int y, int width, int height,
-		     int r, int g, int b)
-{
-	cairo_pattern_t *pat;
-
-	pat = cairo_pattern_create_linear(x, y, x + width, y + height);
-	cairo_pattern_add_color_stop_rgba(pat, 1, 0, 0, 0, 1);
-	cairo_pattern_add_color_stop_rgba(pat, 0, r, g, b, 1);
-
-	cairo_rectangle(cr, x, y, width, height);
-	cairo_set_source(cr, pat);
-	cairo_fill(cr);
-	cairo_pattern_destroy(pat);
-}
-
 static void
 paint_color_key(void)
 {
@@ -495,86 +338,41 @@ paint_color_key(void)
 		}
 }
 
-static void
-paint_test_patterns(cairo_t *cr, int width, int height, int stride)
+static void paint_image(cairo_t *cr, const char *file)
 {
-	double gr_height, gr_width;
-	int x, y;
+	int img_x, img_y, img_w, img_h, img_w_o, img_h_o;
+	double img_w_scale, img_h_scale;
 
-	y = height * 0.10;
-	gr_width = width * 0.75;
-	gr_height = height * 0.08;
-	x = (width / 2) - (gr_width / 2);
+	cairo_surface_t *image;
 
-	paint_color_gradient(cr, x, y, gr_width, gr_height, 1, 0, 0);
+	img_y = height * (0.10 );
+	img_h = height * 0.08 * 4;
+	img_w = img_h;
 
-	y += gr_height;
-	paint_color_gradient(cr, x, y, gr_width, gr_height, 0, 1, 0);
+	img_x = (width / 2) - (img_w / 2);
 
-	y += gr_height;
-	paint_color_gradient(cr, x, y, gr_width, gr_height, 0, 0, 1);
+	image = cairo_image_surface_create_from_png(file);
 
-	y += gr_height;
-	paint_color_gradient(cr, x, y, gr_width, gr_height, 1, 1, 1);
+	img_w_o = cairo_image_surface_get_width(image);
+	img_h_o = cairo_image_surface_get_height(image);
+
+	cairo_translate(cr, img_x, img_y);
+
+	img_w_scale = (double)img_w / (double)img_w_o;
+	img_h_scale = (double)img_h / (double)img_h_o;
+	cairo_scale(cr, img_w_scale, img_h_scale);
+
+	cairo_set_source_surface(cr, image, 0, 0);
+	cairo_scale(cr, 1, 1);
+
+	cairo_paint(cr);
+	cairo_surface_destroy(image);
 }
 
 static void
-paint_marker(cairo_t *cr, int x, int y, char *str, enum corner text_location)
-{
-	cairo_text_extents_t extents;
-	int xoff, yoff;
-
-	cairo_set_font_size(cr, 18);
-	cairo_text_extents(cr, str, &extents);
-
-	switch (text_location) {
-	case topleft:
-		xoff = -20;
-		xoff -= extents.width;
-		yoff = -20;
-		break;
-	case topright:
-		xoff = 20;
-		yoff = -20;
-		break;
-	case bottomleft:
-		xoff = -20;
-		xoff -= extents.width;
-		yoff = 20;
-		break;
-	case bottomright:
-		xoff = 20;
-		yoff = 20;
-		break;
-	default:
-		xoff = 0;
-		yoff = 0;
-	}
-
-	cairo_move_to(cr, x, y - 20);
-	cairo_line_to(cr, x, y + 20);
-	cairo_move_to(cr, x - 20, y);
-	cairo_line_to(cr, x + 20, y);
-	cairo_new_sub_path(cr);
-	cairo_arc(cr, x, y, 10, 0, M_PI * 2);
-	cairo_set_line_width(cr, 4);
-	cairo_set_source_rgb(cr, 0, 0, 0);
-	cairo_stroke_preserve(cr);
-	cairo_set_source_rgb(cr, 1, 1, 1);
-	cairo_set_line_width(cr, 2);
-	cairo_stroke(cr);
-
-	cairo_move_to(cr, x + xoff, y + yoff);
-	cairo_text_path(cr, str);
-	cairo_set_source_rgb(cr, 0, 0, 0);
-	cairo_stroke_preserve(cr);
-	cairo_set_source_rgb(cr, 1, 1, 1);
-	cairo_fill(cr);
-}
-
-static void
-paint_output_info(cairo_t *cr, struct connector *c, int width, int height)
+paint_output_info(cairo_t *cr, int l_width, int l_height, void *priv)
 {
+	struct connector *c = priv;
 	cairo_text_extents_t name_extents, mode_extents;
 	char name_buf[128], mode_buf[128];
 	int i, x, y, modes_x, modes_y;
@@ -595,9 +393,9 @@ paint_output_info(cairo_t *cr, struct connector *c, int width, int height)
 	cairo_text_extents(cr, mode_buf, &mode_extents);
 
 	/* Paint output name */
-	x = width / 2;
+	x = l_width / 2;
 	x -= name_extents.width / 2;
-	y = height / 2;
+	y = l_height / 2;
 	y -= (name_extents.height / 2) - (mode_extents.height / 2) - 10;
 	cairo_set_font_size(cr, 48);
 	cairo_move_to(cr, x, y);
@@ -608,10 +406,10 @@ paint_output_info(cairo_t *cr, struct connector *c, int width, int height)
 	cairo_fill(cr);
 
 	/* Paint mode name */
-	x = width / 2;
+	x = l_width / 2;
 	x -= mode_extents.width / 2;
 	modes_x = x;
-	y = height / 2;
+	y = l_height / 2;
 	y += (mode_extents.height / 2) + (name_extents.height / 2) + 10;
 	cairo_set_font_size(cr, 36);
 	cairo_move_to(cr, x, y);
@@ -656,201 +454,32 @@ paint_output_info(cairo_t *cr, struct connector *c, int width, int height)
 		cairo_set_source_rgb(cr, 1, 1, 1);
 		cairo_fill(cr);
 	}
+
+	if (qr_code)
+		paint_image(cr, "./pass.png");
 }
 
-#ifdef TEST_PLANES
-static int
-connector_find_plane(struct connector *c)
+static void sighandler(int signo)
 {
-	drmModePlaneRes *plane_resources;
-	drmModePlane *ovr;
-	uint32_t id = 0;
-	int i;
-
-	plane_resources = drmModeGetPlaneResources(drm_fd);
-	if (!plane_resources) {
-		fprintf(stderr, "drmModeGetPlaneResources failed: %s\n",
-			strerror(errno));
-		return 0;
-	}
-
-	for (i = 0; i < plane_resources->count_planes; i++) {
-		ovr = drmModeGetPlane(drm_fd, plane_resources->planes[i]);
-		if (!ovr) {
-			fprintf(stderr, "drmModeGetPlane failed: %s\n",
-				strerror(errno));
-			continue;
-		}
-
-		if (ovr->possible_crtcs & (1 << c->pipe)) {
-			id = ovr->plane_id;
-			drmModeFreePlane(ovr);
-			break;
-		}
-		drmModeFreePlane(ovr);
-	}
-
-	return id;
-}
-
-static void
-paint_plane(cairo_t *cr, int width, int height, int stride)
-{
-	double gr_height, gr_width;
-	int x, y;
-
-	y = 0;
-	gr_width = width;
-	gr_height = height * 0.25;
-	x = 0;
-
-	paint_color_gradient(cr, x, y, gr_width, gr_height, 1, 0, 0);
-
-	y += gr_height;
-	paint_color_gradient(cr, x, y, gr_width, gr_height, 0, 1, 0);
-
-	y += gr_height;
-	paint_color_gradient(cr, x, y, gr_width, gr_height, 0, 0, 1);
-
-	y += gr_height;
-	paint_color_gradient(cr, x, y, gr_width, gr_height, 1, 1, 1);
-}
-
-static void
-enable_plane(struct connector *c)
-{
-	cairo_surface_t *surface;
-	cairo_status_t status;
-	cairo_t *cr;
-	uint32_t handle;
-	int ret;
-	uint32_t handles[4], pitches[4], offsets[4]; /* we only use [0] */
-	struct drm_intel_sprite_colorkey set;
-	uint32_t plane_flags = 0;
-
-	plane_id = connector_find_plane(c);
-	if (!plane_id) {
-		fprintf(stderr, "failed to find plane for crtc\n");
-		return;
-	}
-	plane_crtc_id = c->crtc;
-
-	surface = allocate_surface(drm_fd, plane_width, plane_height, 24, &handle, 1);
-	if (!surface) {
-		fprintf(stderr, "allocation failed %dx%d\n", plane_width, plane_height);
-		return;
-	}
-
-	cr = cairo_create(surface);
-
-	paint_plane(cr, plane_width, plane_height,
-		      cairo_image_surface_get_stride(surface));
-	status = cairo_status(cr);
-	cairo_destroy(cr);
-	if (status)
-		fprintf(stderr, "failed to draw plane %dx%d: %s\n",
-			plane_width, plane_height, cairo_status_to_string(status));
-
-	pitches[0] = cairo_image_surface_get_stride(surface);
-	memset(offsets, 0, sizeof(offsets));
-	handles[0] = handles[1] = handles[2] = handles[3] = handle;
-	ret = drmModeAddFB2(drm_fd, plane_width, plane_height, DRM_FORMAT_XRGB8888,
-			    handles, pitches, offsets, &plane_fb_id,
-			    plane_flags);
-	cairo_surface_destroy(surface);
-	gem_close(drm_fd, handle);
-
-	if (ret) {
-		fprintf(stderr, "failed to add fb (%dx%d): %s\n",
-			plane_width, plane_height, strerror(errno));
-		return;
-	}
-
-	set.plane_id = plane_id;
-	set.max_value = SPRITE_COLOR_KEY;
-	set.min_value = SPRITE_COLOR_KEY;
-	set.channel_mask = 0xffffff;
-	ret = drmCommandWrite(drm_fd, DRM_I915_SET_SPRITE_COLORKEY, &set,
-			      sizeof(set));
-
-	if (drmModeSetPlane(drm_fd, plane_id, plane_crtc_id, plane_fb_id,
-			    plane_flags, crtc_x, crtc_y, crtc_w, crtc_h,
-			    0, 0, plane_width, plane_height)) {
-		fprintf(stderr, "failed to enable plane: %s\n",
-			strerror(errno));
-		return;
-	}
-}
-
-static void
-adjust_plane(int fd, int xdistance, int ydistance, int wdiff, int hdiff)
-{
-	uint32_t plane_flags = 0;
-
-	crtc_x += xdistance;
-	crtc_y += ydistance;
-	crtc_w += wdiff;
-	crtc_h += hdiff;
-	fprintf(stderr, "setting plane %dx%d @ %d,%d (source %dx%d)\n",
-		crtc_w, crtc_h, crtc_x, crtc_y, plane_width, plane_height);
-	if (drmModeSetPlane(fd, plane_id, plane_crtc_id, plane_fb_id,
-			    plane_flags, crtc_x, crtc_y,
-			    crtc_w, crtc_h, 0, 0, plane_width, plane_height))
-		fprintf(stderr, "failed to adjust plane: %s\n",	strerror(errno));
-}
-
-static void
-disable_planes(int fd)
-{
-	struct connector *connectors;
-	int c;
-
-	resources = drmModeGetResources(fd);
-	if (!resources) {
-		fprintf(stderr, "drmModeGetResources failed: %s\n",
-			strerror(errno));
-		return;
-	}
-
-	connectors = calloc(resources->count_connectors,
-			    sizeof(struct connector));
-	if (!connectors)
-		return;
-
-	/* Find any connected displays */
-	for (c = 0; c < resources->count_connectors; c++) {
-		uint32_t plane_id;
-
-		plane_id = connector_find_plane(&connectors[c]);
-		if (!plane_id) {
-			fprintf(stderr,
-				"failed to find plane for crtc\n");
-			return;
-		}
-		if (drmModeSetPlane(fd, plane_id, connectors[c].crtc, 0, 0, 0,
-				    0, 0, 0, 0, 0, 0, 0)) {
-			fprintf(stderr, "failed to disable plane: %s\n",
-				strerror(errno));
-			return;
-		}
-	}
-	drmModeFreeResources(resources);
 	return;
 }
-#else
-static void enable_plane(struct connector *c) { return; }
-static void
-adjust_plane(int fd, int xdistance, int ydistance, int wdiff, int hdiff)
-{ return; }
-static void disable_planes(int fd) { return; }
-#endif
+
+static void set_single(void)
+{
+	int sigs[] = { SIGUSR1 };
+	struct sigaction sa;
+	sa.sa_handler = sighandler;
+
+	sigemptyset(&sa.sa_mask);
+
+	if (sigaction(sigs[0], &sa, NULL) == -1)
+		perror("Could not set signal handler");
+}
 
 static void
 set_mode(struct connector *c)
 {
-	unsigned int fb_id;
-	int ret;
-	char buf[128];
+	unsigned int fb_id = 0;
 	int j, test_mode_num;
 
 	if (depth <= 8)
@@ -874,10 +503,7 @@ set_mode(struct connector *c)
 		test_mode_num = c->connector->count_modes;
 
 	for (j = 0; j < test_mode_num; j++) {
-		cairo_surface_t *surface;
-		cairo_status_t status;
-		cairo_t *cr;
-		uint32_t handle;
+		struct kmstest_fb fb_info;
 
 		if (test_all_modes)
 			c->mode = c->connector->modes[j];
@@ -888,55 +514,19 @@ set_mode(struct connector *c)
 		width = c->mode.hdisplay;
 		height = c->mode.vdisplay;
 
-		surface = allocate_surface(drm_fd, width, height, depth,
-					   &handle, enable_tiling);
-		if (!surface) {
-			fprintf(stderr, "allocation failed %dx%d\n", width, height);
-			continue;
-		}
-
-		cr = cairo_create(surface);
-
-		paint_test_patterns(cr, width, height,
-				    cairo_image_surface_get_stride(surface));
-
-		cairo_set_line_cap(cr, CAIRO_LINE_CAP_SQUARE);
-
-		/* Paint corner markers */
-		snprintf(buf, sizeof buf, "(%d, %d)", 0, 0);
-		paint_marker(cr, 0, 0, buf, bottomright);
-		snprintf(buf, sizeof buf, "(%d, %d)", width, 0);
-		paint_marker(cr, width, 0, buf, bottomleft);
-		snprintf(buf, sizeof buf, "(%d, %d)", 0, height);
-		paint_marker(cr, 0, height, buf, topright);
-		snprintf(buf, sizeof buf, "(%d, %d)", width, height);
-		paint_marker(cr, width, height, buf, topleft);
-
-		/* Paint output info */
-		paint_output_info(cr, c, width, height);
+		fb_id = kmstest_create_fb(drm_fd, width, height, bpp, depth,
+					  enable_tiling, &fb_info,
+					  paint_output_info, c);
 
+		fb_ptr = gem_mmap(drm_fd, fb_info.gem_handle,
+				  fb_info.size, PROT_READ | PROT_WRITE);
+		assert(fb_ptr);
 		paint_color_key();
 
-		status = cairo_status(cr);
-		cairo_destroy(cr);
-		if (status)
-			fprintf(stderr, "failed to draw pretty picture %dx%d: %s\n",
-				width, height, cairo_status_to_string(status));
-
-		ret = drmModeAddFB(drm_fd, width, height, depth, bpp,
-				   cairo_image_surface_get_stride(surface),
-				   handle, &fb_id);
-		cairo_surface_destroy(surface);
-		gem_close(drm_fd, handle);
-
-		if (ret) {
-			fprintf(stderr, "failed to add fb (%dx%d): %s\n",
-				width, height, strerror(errno));
-			continue;
-		}
+		gem_close(drm_fd, fb_info.gem_handle);
 
 		fprintf(stdout, "CRTS(%u):",c->crtc);
-		dump_mode(&c->mode);
+		kmstest_dump_mode(&c->mode);
 		if (drmModeSetCrtc(drm_fd, c->crtc, fb_id, 0, 0,
 				   &c->id, 1, &c->mode)) {
 			fprintf(stderr, "failed to set mode (%dx%d@%dHz): %s\n",
@@ -945,12 +535,14 @@ set_mode(struct connector *c)
 			continue;
 		}
 
-		if (test_plane)
-			enable_plane(c);
-
-		if (sleep_between_modes && test_all_modes)
+		if (sleep_between_modes && test_all_modes && !qr_code)
 			sleep(sleep_between_modes);
 
+		if (qr_code){
+			set_single();
+			pause();
+		}
+
 	}
 
 	if(test_all_modes){
@@ -991,7 +583,6 @@ int update_display(void)
 	if (dump_info) {
 		dump_connectors_fd(drm_fd);
 		dump_crtcs_fd(drm_fd);
-		dump_planes();
 	}
 
 	if (test_preferred_mode || test_all_modes || force_mode) {
@@ -1005,7 +596,7 @@ int update_display(void)
 	return 1;
 }
 
-static char optstr[] = "hiaf:s:d:p:mt";
+static char optstr[] = "hiaf:s:d:p:mrt";
 
 static void usage(char *name)
 {
@@ -1017,6 +608,7 @@ static void usage(char *name)
 	fprintf(stderr, "\t-p\t<planew,h>,<crtcx,y>,<crtcw,h> test overlay plane\n");
 	fprintf(stderr, "\t-m\ttest the preferred mode\n");
 	fprintf(stderr, "\t-t\tuse a tiled framebuffer\n");
+	fprintf(stderr, "\t-r\tprint a QR code on the screen whose content is \"pass\" for the automatic test\n");
 	fprintf(stderr, "\t-f\t<clock MHz>,<hdisp>,<hsync-start>,<hsync-end>,<htotal>,\n");
 	fprintf(stderr, "\t\t<vdisp>,<vsync-start>,<vsync-end>,<vtotal>\n");
 	fprintf(stderr, "\t\ttest force mode\n");
@@ -1027,45 +619,47 @@ static void usage(char *name)
 #define dump_resource(res) if (res) dump_##res()
 
 static gboolean input_event(GIOChannel *source, GIOCondition condition,
-			    gpointer data)
+				gpointer data)
 {
 	gchar buf[2];
 	gsize count;
 
 	count = read(g_io_channel_unix_get_fd(source), buf, sizeof(buf));
 	if (buf[0] == 'q' && (count == 1 || buf[1] == '\n')) {
-		disable_planes(drm_fd);
 		exit(0);
-	} else if (buf[0] == 'a')
-		adjust_plane(drm_fd, -10, 0, 0, 0);
-	else if (buf[0] == 'd')
-		adjust_plane(drm_fd, 10, 0, 0, 0);
-	else if (buf[0] == 'w')
-		adjust_plane(drm_fd, 0, -10, 0, 0);
-	else if (buf[0] == 's')
-		adjust_plane(drm_fd, 0, 10, 0, 0);
-	else if (buf[0] == 'j')
-		adjust_plane(drm_fd, 0, 0, 10, 0);
-	else if (buf[0] == 'l')
-		adjust_plane(drm_fd, 0, 0, -10, 0);
-	else if (buf[0] == 'k')
-		adjust_plane(drm_fd, 0, 0, 0, -10);
-	else if (buf[0] == 'i')
-		adjust_plane(drm_fd, 0, 0, 0, 10);
+	}
 
 	return TRUE;
 }
 
+static void enter_exec_path( char **argv )
+{
+	char *exec_path = NULL;
+	char *pos = NULL;
+	short len_path = 0;
+
+	len_path = strlen( argv[0] );
+	exec_path = (char*) malloc(len_path);
+
+	memcpy(exec_path, argv[0], len_path);
+	pos = strrchr(exec_path, '/');
+	if (pos != NULL)
+		*(pos+1) = '\0';
+
+	chdir(exec_path);
+	free(exec_path);
+}
+
 int main(int argc, char **argv)
 {
 	int c;
-	const char *modules[] = { "i915" };
-	unsigned int i;
 	int ret = 0;
 	GIOChannel *stdinchannel;
 	GMainLoop *mainloop;
 	float force_clock;
 
+	enter_exec_path( argv );
+
 	opterr = 0;
 	while ((c = getopt(argc, argv, optstr)) != -1) {
 		switch (c) {
@@ -1104,6 +698,9 @@ int main(int argc, char **argv)
 		case 't':
 			enable_tiling = 1;
 			break;
+		case 'r':
+			qr_code = 1;
+			break;
 		default:
 			fprintf(stderr, "unknown option %c\n", c);
 			/* fall through */
@@ -1116,19 +713,7 @@ int main(int argc, char **argv)
 	    !test_preferred_mode)
 		test_all_modes = 1;
 
-	for (i = 0; i < ARRAY_SIZE(modules); i++) {
-		drm_fd = drmOpen(modules[i], NULL);
-		if (drm_fd < 0)
-			printf("failed to load %s driver.\n", modules[i]);
-		else
-			break;
-	}
-
-	if (i == ARRAY_SIZE(modules)) {
-		fprintf(stderr, "failed to load any modules, aborting.\n");
-		ret = -1;
-		goto out;
-	}
+	drm_fd = drm_open_any();
 
 	mainloop = g_main_loop_new(NULL, FALSE);
 	if (!mainloop) {
@@ -1174,9 +759,7 @@ out_hotplug:
 out_mainloop:
 	g_main_loop_unref(mainloop);
 out_close:
-	if (test_plane)
-		disable_planes(drm_fd);
-	drmClose(drm_fd);
-out:
+	close(drm_fd);
+
 	return ret;
 }
