@@ -30,9 +30,10 @@
 #include <stdio.h>
 #include <string.h>
 #include <err.h>
+#include <unistd.h>
 #include "intel_gpu_tools.h"
 
-static uint32_t devid;
+static uint32_t devid = 0;
 
 #define DEBUGSTRING(func) static void func(char *result, int len, int reg, uint32_t val)
 
@@ -141,7 +142,7 @@ DEBUGSTRING(i830_debug_dspcntr)
 DEBUGSTRING(i830_debug_pipeconf)
 {
 	const char *enabled = val & PIPEACONF_ENABLE ? "enabled" : "disabled";
-	const char *bit30;
+	const char *bit30, *interlace;
 
 	if (IS_965(devid))
 		bit30 = val & I965_PIPECONF_ACTIVE ? "active" : "inactive";
@@ -150,7 +151,43 @@ DEBUGSTRING(i830_debug_pipeconf)
 		    val & PIPEACONF_DOUBLE_WIDE ? "double-wide" : "single-wide";
 
 	if (HAS_PCH_SPLIT(devid)) {
-		const char *bpc;
+		const char *bpc, *rotation;
+
+		switch ((val >> 21) & 7) {
+		case 0:
+			interlace = "pf-pd";
+			break;
+		case 1:
+			interlace = "pf-id";
+			break;
+		case 3:
+			interlace = "if-id";
+			break;
+		case 4:
+			interlace = "if-id-dbl";
+			break;
+		case 5:
+			interlace = "pf-id-dbl";
+			break;
+		default:
+			interlace = "rsvd";
+			break;
+		}
+
+		switch ((val >> 14) & 3) {
+		case 0:
+			rotation = "rotate 0";
+			break;
+		case 1:
+			rotation = "rotate 90";
+			break;
+		case 2:
+			rotation = "rotate 180";
+			break;
+		case 3:
+			rotation = "rotate 270";
+			break;
+		}
 
 		switch (val & (7 << 5)) {
 		case PIPECONF_8BPP:
@@ -169,7 +206,30 @@ DEBUGSTRING(i830_debug_pipeconf)
 			bpc = "invalid bpc";
 			break;
 		}
-		snprintf(result, len, "%s, %s, %s", enabled, bit30, bpc);
+		snprintf(result, len, "%s, %s, %s, %s, %s", enabled, bit30,
+			 interlace, rotation, bpc);
+	} else if (IS_GEN4(devid)) {
+		switch ((val >> 21) & 7) {
+		case 0:
+		case 1:
+		case 2:
+		case 3:
+			interlace = "progressive";
+			break;
+		case 4:
+			interlace = "interlaced embedded";
+			break;
+		case 5:
+			interlace = "interlaced";
+			break;
+		case 6:
+			interlace = "interlaced sdvo";
+			break;
+		case 7:
+			interlace = "interlaced legacy";
+			break;
+		}
+		snprintf(result, len, "%s, %s, %s", enabled, bit30, interlace);
 	} else
 		snprintf(result, len, "%s, %s", enabled, bit30);
 }
@@ -463,25 +523,25 @@ DEBUGSTRING(i830_debug_dpll_test)
 
 DEBUGSTRING(i830_debug_adpa)
 {
-	char pipe = (val & ADPA_PIPE_B_SELECT) ? 'B' : 'A';
+	char disp_pipe = (val & ADPA_PIPE_B_SELECT) ? 'B' : 'A';
 	const char *enable = (val & ADPA_DAC_ENABLE) ? "enabled" : "disabled";
 	char hsync = (val & ADPA_HSYNC_ACTIVE_HIGH) ? '+' : '-';
 	char vsync = (val & ADPA_VSYNC_ACTIVE_HIGH) ? '+' : '-';
 
 	if (HAS_CPT)
-		pipe = val & (1<<29) ? 'B' : 'A';
+		disp_pipe = val & (1<<29) ? 'B' : 'A';
 
 	if (HAS_PCH_SPLIT(devid))
 		snprintf(result, len, "%s, transcoder %c, %chsync, %cvsync",
-				 enable, pipe, hsync, vsync);
+				 enable, disp_pipe, hsync, vsync);
 	else
 		snprintf(result, len, "%s, pipe %c, %chsync, %cvsync",
-				 enable, pipe, hsync, vsync);
+				 enable, disp_pipe, hsync, vsync);
 }
 
 DEBUGSTRING(i830_debug_lvds)
 {
-	char pipe = val & LVDS_PIPEB_SELECT ? 'B' : 'A';
+	char disp_pipe = val & LVDS_PIPEB_SELECT ? 'B' : 'A';
 	const char *enable = val & LVDS_PORT_EN ? "enabled" : "disabled";
 	int depth;
 	const char *channels;
@@ -496,16 +556,16 @@ DEBUGSTRING(i830_debug_lvds)
 		channels = "1 channel";
 
 	if (HAS_CPT)
-		pipe = val & (1<<29) ? 'B' : 'A';
+		disp_pipe = val & (1<<29) ? 'B' : 'A';
 
 	snprintf(result, len, "%s, pipe %c, %d bit, %s",
-			 enable, pipe, depth, channels);
+			 enable, disp_pipe, depth, channels);
 }
 
 DEBUGSTRING(i830_debug_dvo)
 {
 	const char *enable = val & DVO_ENABLE ? "enabled" : "disabled";
-	char pipe = val & DVO_PIPE_B_SELECT ? 'B' : 'A';
+	char disp_pipe = val & DVO_PIPE_B_SELECT ? 'B' : 'A';
 	const char *stall;
 	char hsync = val & DVO_HSYNC_ACTIVE_HIGH ? '+' : '-';
 	char vsync = val & DVO_VSYNC_ACTIVE_HIGH ? '+' : '-';
@@ -526,13 +586,13 @@ DEBUGSTRING(i830_debug_dvo)
 	}
 
 	snprintf(result, len, "%s, pipe %c, %s, %chsync, %cvsync",
-			 enable, pipe, stall, hsync, vsync);
+			 enable, disp_pipe, stall, hsync, vsync);
 }
 
 DEBUGSTRING(i830_debug_sdvo)
 {
 	const char *enable = val & SDVO_ENABLE ? "enabled" : "disabled";
-	char pipe = val & SDVO_PIPE_B_SELECT ? 'B' : 'A';
+	char disp_pipe = val & SDVO_PIPE_B_SELECT ? 'B' : 'A';
 	const char *stall = val & SDVO_STALL_SELECT ? "enabled" : "disabled";
 	const char *detected = val & SDVO_DETECTED ? "" : "not ";
 	const char *gang = val & SDVOC_GANG_MODE ? ", gang mode" : "";
@@ -547,7 +607,7 @@ DEBUGSTRING(i830_debug_sdvo)
 	}
 
 	snprintf(result, len, "%s, pipe %c, stall %s, %sdetected%s%s",
-			 enable, pipe, stall, detected, sdvoextra, gang);
+			 enable, disp_pipe, stall, detected, sdvoextra, gang);
 }
 
 DEBUGSTRING(i830_debug_dspclk_gate_d)
@@ -700,6 +760,9 @@ static struct reg_debug intel_debug_regs[] = {
 	DEFINEREG(DVOB_SRCDIM),
 	DEFINEREG(DVOC_SRCDIM),
 
+	DEFINEREG(BLC_PWM_CTL),
+	DEFINEREG(BLC_PWM_CTL2),
+
 	DEFINEREG2(PP_CONTROL, i830_debug_pp_control),
 	DEFINEREG2(PP_STATUS, i830_debug_pp_status),
 	DEFINEREG(PP_ON_DELAYS),
@@ -1249,9 +1312,28 @@ DEBUGSTRING(ironlake_debug_fdi_rx_misc)
 
 DEBUGSTRING(ironlake_debug_transconf)
 {
-	snprintf(result, len, "%s, %s",
-		 val & TRANS_ENABLE ? "enable" : "disable",
-		 val & TRANS_STATE_ENABLE ? "active" : "inactive");
+	const char *enable = val & TRANS_ENABLE ? "enable" : "disable";
+	const char *state = val & TRANS_STATE_ENABLE ? "active" : "inactive";
+	const char *interlace;
+
+	switch ((val >> 21) & 7) {
+	case 0:
+		interlace = "progressive";
+		break;
+	case 2:
+		if (IS_GEN5(devid))
+			interlace = "interlaced sdvo";
+		else
+			interlace = "rsvd";
+		break;
+	case 3:
+		interlace = "interlaced";
+		break;
+	default:
+		interlace = "rsvd";
+	}
+
+	snprintf(result, len, "%s, %s, %s", enable, state, interlace);
 }
 
 DEBUGSTRING(ironlake_debug_panel_fitting)
@@ -1336,7 +1418,7 @@ DEBUGSTRING(ironlake_debug_pf_win)
 
 DEBUGSTRING(ironlake_debug_hdmi)
 {
-	int pipe;
+	int disp_pipe;
 	const char *enable, *bpc = NULL, *encoding;
 	const char *mode, *audio, *vsync, *hsync, *detect;
 
@@ -1346,9 +1428,9 @@ DEBUGSTRING(ironlake_debug_hdmi)
 		enable = "disabled";
 
 	if (HAS_CPT)
-		pipe = (val & (3<<29)) >> 29;
+		disp_pipe = (val & (3<<29)) >> 29;
 	else
-		pipe = (val & TRANSCODER_B) >> 29;
+		disp_pipe = (val & TRANSCODER_B) >> 29;
 
 	switch (val & (7 << 26)) {
 	case COLOR_FORMAT_8bpc:
@@ -1390,7 +1472,7 @@ DEBUGSTRING(ironlake_debug_hdmi)
 		detect = "non-detected";
 
 	snprintf(result, len, "%s pipe %c %s %s %s audio %s %s %s %s",
-		 enable, pipe + 'A', bpc, encoding, mode, audio, vsync, hsync, detect);
+		 enable, disp_pipe + 'A', bpc, encoding, mode, audio, vsync, hsync, detect);
 }
 
 DEBUGSTRING(snb_debug_dpll_sel)
@@ -1506,6 +1588,8 @@ static struct reg_debug ironlake_debug_regs[] = {
 
 	DEFINEREG(FDI_PLL_FREQ_CTL),
 
+	/* pipe B */
+
 	DEFINEREG2(PIPEACONF, i830_debug_pipeconf),
 
 	DEFINEREG2(HTOTAL_A, i830_debug_hvtotal),
@@ -1533,6 +1617,8 @@ static struct reg_debug ironlake_debug_regs[] = {
 	DEFINEREG(DSPASURF),
 	DEFINEREG2(DSPATILEOFF, i830_debug_xy),
 
+	/* pipe B */
+
 	DEFINEREG2(PIPEBCONF, i830_debug_pipeconf),
 
 	DEFINEREG2(HTOTAL_B, i830_debug_hvtotal),
@@ -1542,13 +1628,6 @@ static struct reg_debug ironlake_debug_regs[] = {
 	DEFINEREG2(VBLANK_B, i830_debug_hvsyncblank),
 	DEFINEREG2(VSYNC_B, i830_debug_hvsyncblank),
 	DEFINEREG(VSYNCSHIFT_B),
-
-	DEFINEREG2(DSPBCNTR, i830_debug_dspcntr),
-	DEFINEREG(DSPBBASE),
-	DEFINEREG2(DSPBSTRIDE, ironlake_debug_dspstride),
-	DEFINEREG(DSPBSURF),
-	DEFINEREG2(DSPBTILEOFF, i830_debug_xy),
-
 	DEFINEREG2(PIPEBSRC, i830_debug_yxminus1),
 
 	DEFINEREG2(PIPEB_DATA_M1, ironlake_debug_m_tu),
@@ -1561,6 +1640,43 @@ static struct reg_debug ironlake_debug_regs[] = {
 	DEFINEREG2(PIPEB_LINK_M2, ironlake_debug_n),
 	DEFINEREG2(PIPEB_LINK_N2, ironlake_debug_n),
 
+	DEFINEREG2(DSPBCNTR, i830_debug_dspcntr),
+	DEFINEREG(DSPBBASE),
+	DEFINEREG2(DSPBSTRIDE, ironlake_debug_dspstride),
+	DEFINEREG(DSPBSURF),
+	DEFINEREG2(DSPBTILEOFF, i830_debug_xy),
+
+	/* pipe C */
+
+	DEFINEREG2(PIPECCONF, i830_debug_pipeconf),
+
+	DEFINEREG2(HTOTAL_C, i830_debug_hvtotal),
+	DEFINEREG2(HBLANK_C, i830_debug_hvsyncblank),
+	DEFINEREG2(HSYNC_C, i830_debug_hvsyncblank),
+	DEFINEREG2(VTOTAL_C, i830_debug_hvtotal),
+	DEFINEREG2(VBLANK_C, i830_debug_hvsyncblank),
+	DEFINEREG2(VSYNC_C, i830_debug_hvsyncblank),
+	DEFINEREG(VSYNCSHIFT_C),
+	DEFINEREG2(PIPECSRC, i830_debug_yxminus1),
+
+	DEFINEREG2(PIPEC_DATA_M1, ironlake_debug_m_tu),
+	DEFINEREG2(PIPEC_DATA_N1, ironlake_debug_n),
+	DEFINEREG2(PIPEC_DATA_M2, ironlake_debug_m_tu),
+	DEFINEREG2(PIPEC_DATA_N2, ironlake_debug_n),
+
+	DEFINEREG2(PIPEC_LINK_M1, ironlake_debug_n),
+	DEFINEREG2(PIPEC_LINK_N1, ironlake_debug_n),
+	DEFINEREG2(PIPEC_LINK_M2, ironlake_debug_n),
+	DEFINEREG2(PIPEC_LINK_N2, ironlake_debug_n),
+
+	DEFINEREG2(DSPCCNTR, i830_debug_dspcntr),
+	DEFINEREG(DSPCBASE),
+	DEFINEREG2(DSPCSTRIDE, ironlake_debug_dspstride),
+	DEFINEREG(DSPCSURF),
+	DEFINEREG2(DSPCTILEOFF, i830_debug_xy),
+
+	/* Panel fitter */
+
 	DEFINEREG2(PFA_CTL_1, ironlake_debug_panel_fitting),
 	DEFINEREG2(PFA_CTL_2, ironlake_debug_panel_fitting_2),
 	DEFINEREG2(PFA_CTL_3, ironlake_debug_panel_fitting_3),
@@ -1573,6 +1689,12 @@ static struct reg_debug ironlake_debug_regs[] = {
 	DEFINEREG2(PFB_CTL_4, ironlake_debug_panel_fitting_4),
 	DEFINEREG2(PFB_WIN_POS, ironlake_debug_pf_win),
 	DEFINEREG2(PFB_WIN_SIZE, ironlake_debug_pf_win),
+	DEFINEREG2(PFC_CTL_1, ironlake_debug_panel_fitting),
+	DEFINEREG2(PFC_CTL_2, ironlake_debug_panel_fitting_2),
+	DEFINEREG2(PFC_CTL_3, ironlake_debug_panel_fitting_3),
+	DEFINEREG2(PFC_CTL_4, ironlake_debug_panel_fitting_4),
+	DEFINEREG2(PFC_WIN_POS, ironlake_debug_pf_win),
+	DEFINEREG2(PFC_WIN_SIZE, ironlake_debug_pf_win),
 
 	/* PCH */
 
@@ -1702,6 +1824,71 @@ static struct reg_debug ironlake_debug_regs[] = {
 	DEFINEREG(RC6pp_RESIDENCY_TIME),
 };
 
+static struct reg_debug haswell_debug_regs[] = {
+	/* Power wells */
+	DEFINEREG(HSW_PWR_WELL_CTL1),
+	DEFINEREG(HSW_PWR_WELL_CTL2),
+	DEFINEREG(HSW_PWR_WELL_CTL3),
+	DEFINEREG(HSW_PWR_WELL_CTL4),
+	DEFINEREG(HSW_PWR_WELL_CTL5),
+	DEFINEREG(HSW_PWR_WELL_CTL6),
+
+	/* DDI pipe function */
+	DEFINEREG(PIPE_DDI_FUNC_CTL_A),
+	DEFINEREG(PIPE_DDI_FUNC_CTL_B),
+	DEFINEREG(PIPE_DDI_FUNC_CTL_C),
+	DEFINEREG(PIPE_DDI_FUNC_CTL_EDP),
+
+	/* DP transport control */
+	DEFINEREG(DP_TP_CTL_A),
+	DEFINEREG(DP_TP_CTL_B),
+	DEFINEREG(DP_TP_CTL_C),
+	DEFINEREG(DP_TP_CTL_D),
+	DEFINEREG(DP_TP_CTL_E),
+
+	/* DP status */
+	DEFINEREG(DP_TP_STATUS_A),
+	DEFINEREG(DP_TP_STATUS_B),
+	DEFINEREG(DP_TP_STATUS_C),
+	DEFINEREG(DP_TP_STATUS_D),
+	DEFINEREG(DP_TP_STATUS_E),
+
+	/* DDI buffer control */
+	DEFINEREG(DDI_BUF_CTL_A),
+	DEFINEREG(DDI_BUF_CTL_B),
+	DEFINEREG(DDI_BUF_CTL_C),
+	DEFINEREG(DDI_BUF_CTL_D),
+	DEFINEREG(DDI_BUF_CTL_E),
+
+	/* Clocks */
+	DEFINEREG(PIXCLK_GATE),
+	DEFINEREG(SPLL_CTL),
+	DEFINEREG(LCPLL_CTL),
+	DEFINEREG(WRPLL_CTL1),
+	DEFINEREG(WRPLL_CTL2),
+
+	/* DDI port clock control */
+	DEFINEREG(PORT_CLK_SEL_A),
+	DEFINEREG(PORT_CLK_SEL_B),
+	DEFINEREG(PORT_CLK_SEL_C),
+	DEFINEREG(PORT_CLK_SEL_D),
+	DEFINEREG(PORT_CLK_SEL_E),
+
+	/* Pipe clock control */
+	DEFINEREG(PIPE_CLK_SEL_A),
+	DEFINEREG(PIPE_CLK_SEL_B),
+	DEFINEREG(PIPE_CLK_SEL_C),
+
+	/* Pipe line time */
+	DEFINEREG(PIPE_WM_LINETIME_A),
+	DEFINEREG(PIPE_WM_LINETIME_B),
+	DEFINEREG(PIPE_WM_LINETIME_C),
+
+	/* Fuses */
+	DEFINEREG(SFUSE_STRAP),
+
+};
+
 static struct reg_debug i945gm_mi_regs[] = {
 	DEFINEREG(PGETBL_CTL),
 	DEFINEREG(PGTBL_ER),
@@ -1710,7 +1897,7 @@ static struct reg_debug i945gm_mi_regs[] = {
 	DEFINEREG(IPEIR),
 	DEFINEREG(IPEHR),
 	DEFINEREG(INST_DONE),
-	DEFINEREG(NOPID),
+	DEFINEREG(NOP_ID),
 	DEFINEREG(HWSTAM),
 	DEFINEREG(SCPD0),
 	DEFINEREG(IER),
@@ -1724,61 +1911,65 @@ static struct reg_debug i945gm_mi_regs[] = {
 	DEFINEREG(ECOSKPD),
 };
 
+#define intel_dump_regs(regs) _intel_dump_regs(regs, ARRAY_SIZE(regs))
+
 static void
-i945_dump_mi_regs(void)
+_intel_dump_regs(struct reg_debug *regs, int count)
 {
 	char debug[1024];
 	int i;
 
-	for (i = 0; i < ARRAY_SIZE(i945gm_mi_regs); i++) {
-		uint32_t val = INREG(i945gm_mi_regs[i].reg);
+	for (i = 0; i < count; i++) {
+		uint32_t val = INREG(regs[i].reg);
 
-		if (i945gm_mi_regs[i].debug_output != NULL) {
-			i945gm_mi_regs[i].debug_output(debug, sizeof(debug),
-						       i945gm_mi_regs
-						       [i].reg,
-						       val);
+		if (regs[i].debug_output != NULL) {
+			regs[i].debug_output(debug, sizeof(debug), regs[i].reg, val);
 			printf("%30.30s: 0x%08x (%s)\n",
-			       i945gm_mi_regs[i].name,
+			       regs[i].name,
 			       (unsigned int)val, debug);
 		} else {
-			printf("%30.30s: 0x%08x\n", i945gm_mi_regs[i].name,
+			printf("%30.30s: 0x%08x\n", regs[i].name,
 			       (unsigned int)val);
 		}
 	}
 }
 
-static void
-ironlake_dump_regs(void)
+DEBUGSTRING(gen6_rp_control)
 {
-	char debug[1024];
-	int i;
-
-	for (i = 0; i < ARRAY_SIZE(ironlake_debug_regs); i++) {
-		uint32_t val = INREG(ironlake_debug_regs[i].reg);
-
-		if (ironlake_debug_regs[i].debug_output != NULL) {
-			ironlake_debug_regs[i].debug_output(debug, sizeof(debug),
-							    ironlake_debug_regs
-							    [i].reg,
-							    val);
-			printf("%30.30s: 0x%08x (%s)\n",
-			       ironlake_debug_regs[i].name,
-			       (unsigned int)val, debug);
-		} else {
-			printf("%30.30s: 0x%08x\n", ironlake_debug_regs[i].name,
-			       (unsigned int)val);
-		}
-	}
+	snprintf(result, len, "%s",
+		 (val & (1 << 7)) ? "enabled" : "disabled");
 }
 
+static struct reg_debug gen6_rp_debug_regs[] = {
+	DEFINEREG2(GEN6_RP_CONTROL, gen6_rp_control),
+	DEFINEREG(GEN6_RPNSWREQ),
+	DEFINEREG(GEN6_RP_DOWN_TIMEOUT),
+	DEFINEREG(GEN6_RP_INTERRUPT_LIMITS),
+	DEFINEREG(GEN6_RP_UP_THRESHOLD),
+	DEFINEREG(GEN6_RP_UP_EI),
+	DEFINEREG(GEN6_RP_DOWN_EI),
+	DEFINEREG(GEN6_RP_IDLE_HYSTERSIS),
+	DEFINEREG(GEN6_RC_STATE),
+	DEFINEREG(GEN6_RC_CONTROL),
+	DEFINEREG(GEN6_RC1_WAKE_RATE_LIMIT),
+	DEFINEREG(GEN6_RC6_WAKE_RATE_LIMIT),
+	DEFINEREG(GEN6_RC_EVALUATION_INTERVAL),
+	DEFINEREG(GEN6_RC_IDLE_HYSTERSIS),
+	DEFINEREG(GEN6_RC_SLEEP),
+	DEFINEREG(GEN6_RC1e_THRESHOLD),
+	DEFINEREG(GEN6_RC6_THRESHOLD),
+	DEFINEREG(GEN6_RC_VIDEO_FREQ),
+	DEFINEREG(GEN6_PMIER),
+	DEFINEREG(GEN6_PMIMR),
+	DEFINEREG(GEN6_PMINTRMSK),
+};
+
 static void
-intel_dump_regs(void)
+intel_dump_other_regs(void)
 {
-	char debug[1024];
 	int i;
 	int fp, dpll;
-	int pipe;
+	int disp_pipe;
 	int n, m1, m2, m, p1, p2;
 	int ref;
 	int dot;
@@ -1788,22 +1979,6 @@ intel_dump_regs(void)
 	int crt;
 #endif
 
-	for (i = 0; i < ARRAY_SIZE(intel_debug_regs); i++) {
-		uint32_t val = INREG(intel_debug_regs[i].reg);
-
-		if (intel_debug_regs[i].debug_output != NULL) {
-			intel_debug_regs[i].debug_output(debug, sizeof(debug),
-							 intel_debug_regs[i].reg,
-							 val);
-			printf("%20.20s: 0x%08x (%s)\n",
-			       intel_debug_regs[i].name,
-			       (unsigned int)val, debug);
-		} else {
-			printf("%20.20s: 0x%08x\n",
-			       intel_debug_regs[i].name,
-			       (unsigned int)val);
-		}
-	}
 #if 0
 	i830DumpIndexed(pScrn, "SR", 0x3c4, 0x3c5, 0, 7);
 	msr = INREG8(0x3cc);
@@ -1817,14 +1992,14 @@ intel_dump_regs(void)
 		crt = 0x3b0;
 	i830DumpIndexed(pScrn, "CR", crt + 4, crt + 5, 0, 0x24);
 #endif
-	for (pipe = 0; pipe <= 1; pipe++) {
-		fp = INREG(pipe == 0 ? FPA0 : FPB0);
-		dpll = INREG(pipe == 0 ? DPLL_A : DPLL_B);
+	for (disp_pipe = 0; disp_pipe <= 1; disp_pipe++) {
+		fp = INREG(disp_pipe == 0 ? FPA0 : FPB0);
+		dpll = INREG(disp_pipe == 0 ? DPLL_A : DPLL_B);
 		if (IS_GEN2(devid)) {
 			uint32_t lvds = INREG(LVDS);
 			if (devid == PCI_CHIP_I855_GM &&
 			    (lvds & LVDS_PORT_EN) &&
-			    (lvds & LVDS_PIPEB_SELECT) == (pipe << 30)) {
+			    (lvds & LVDS_PIPEB_SELECT) == (disp_pipe << 30)) {
 				if ((lvds & LVDS_CLKB_POWER_MASK) ==
 				    LVDS_CLKB_POWER_UP)
 					p2 = 7;
@@ -1881,7 +2056,7 @@ intel_dump_regs(void)
 		} else {
 			uint32_t lvds = INREG(LVDS);
 			if ((lvds & LVDS_PORT_EN) &&
-			    (lvds & LVDS_PIPEB_SELECT) == (pipe << 30)) {
+			    (lvds & LVDS_PIPEB_SELECT) == (disp_pipe << 30)) {
 				if ((lvds & LVDS_CLKB_POWER_MASK) ==
 				    LVDS_CLKB_POWER_UP)
 					p2 = 7;
@@ -1990,32 +2165,82 @@ intel_dump_regs(void)
 		}
 
 		printf("pipe %s dot %d n %d m1 %d m2 %d p1 %d p2 %d\n",
-		       pipe == 0 ? "A" : "B", dot, n, m1, m2, p1, p2);
+		       disp_pipe == 0 ? "A" : "B", dot, n, m1, m2, p1, p2);
 	}
 }
 
+static void print_usage(void)
+{
+	printf("Usage: intel_reg_dumper [options] [file]\n"
+	       "Options:\n"
+	       "  -d id   when a dump file is used, use 'id' as device id (in "
+	       "hex)\n"
+	       "  -h      prints this help\n");
+}
+
 int main(int argc, char** argv)
 {
 	struct pci_device *pci_dev;
+	int opt;
+	char *file = NULL;
 
-	if (argc == 2)
-		intel_map_file(argv[1]);
-	else {
+	while ((opt = getopt(argc, argv, "d:h")) != -1) {
+		switch (opt) {
+		case 'd':
+			devid = strtol(optarg, NULL, 16);
+			break;
+		case 'h':
+			print_usage();
+			return 0;
+		default:
+			print_usage();
+			return 1;
+		}
+	}
+	if (optind < argc)
+		file = argv[optind];
+
+	if (file) {
+		intel_map_file(file);
+		if (devid) {
+			if (IS_GEN5(devid))
+				pch = PCH_IBX;
+			else
+				pch = PCH_CPT;
+		} else {
+			printf("Dumping from file without -d argument. "
+			       "Assuming Ironlake machine.\n");
+			devid = 0x0042;
+			pch = PCH_IBX;
+		}
+	} else {
 		pci_dev = intel_get_pci_device();
-		devid = pci_dev->device_id; /* XXX not true when mapping! */
+		devid = pci_dev->device_id;
 
-		intel_get_mmio(pci_dev);
+		intel_register_access_init(pci_dev, 1);
+
+		if (HAS_PCH_SPLIT(devid))
+			intel_check_pch();
 	}
 
-	if (HAS_PCH_SPLIT(devid) || getenv("HAS_PCH_SPLIT")) {
-		intel_check_pch();
-		ironlake_dump_regs();
+	if (HAS_PCH_SPLIT(devid)) {
+		intel_dump_regs(ironlake_debug_regs);
+	} else if (IS_945GM(devid)) {
+		intel_dump_regs(i945gm_mi_regs);
+		intel_dump_regs(intel_debug_regs);
+		intel_dump_other_regs();
+	} else {
+		intel_dump_regs(intel_debug_regs);
+		intel_dump_other_regs();
 	}
-	else if (IS_945GM(devid)) {
-		i945_dump_mi_regs();
-		intel_dump_regs();
-	} else
-		intel_dump_regs();
 
+	if (IS_GEN6(devid) || IS_GEN7(devid))
+		intel_dump_regs(gen6_rp_debug_regs);
+
+	if (IS_HASWELL(devid))
+		intel_dump_regs(haswell_debug_regs);
+
+
+	intel_register_access_fini();
 	return 0;
 }
